@@ -1,7 +1,7 @@
 /*
  * pseudo_db.c, sqlite3 interface
  * 
- * Copyright (c) 2008-2010 Wind River Systems, Inc.
+ * Copyright (c) 2008-2010,2013 Wind River Systems, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the Lesser GNU General Public License version 2.1 as
@@ -50,6 +50,10 @@ struct pdb_file_list {
 	sqlite3_stmt *stmt;
 };
 
+static int file_db_dirty = 0;
+#ifdef USE_MEMORY_DB
+static sqlite3 *real_file_db = 0;
+#endif
 static sqlite3 *file_db = 0;
 static sqlite3 *log_db = 0;
 
@@ -115,6 +119,34 @@ static struct sql_index {
 	{ NULL, NULL, NULL },
 };
 
+static char *file_pragmas[] = {
+	"PRAGMA legacy_file_format = OFF;",
+	"PRAGMA journal_mode = OFF;",
+	/* the default page size produces painfully bad behavior
+	 * for memory databases with some versions of sqlite.
+	 */
+	"PRAGMA page_size = 8192;",
+	"PRAGMA locking_mode = EXCLUSIVE;",
+	/* Setting this to NORMAL makes pseudo noticably slower
+	 * than fakeroot, but is perhaps more secure.  However,
+	 * note that sqlite always flushes to the OS; what is lacking
+	 * in non-synchronous mode is waiting for the OS to
+	 * confirm delivery to media, and also a bunch of cache
+	 * flushing and reloading which we probably don't really
+	 * need.
+	 */
+	"PRAGMA synchronous = OFF;",
+	NULL
+};
+
+static char *log_pragmas[] = {
+	"PRAGMA legacy_file_format = OFF;",
+	"PRAGMA journal_mode = OFF;",
+	"PRAGMA locking_mode = EXCLUSIVE;",
+	"PRAGMA synchronous = OFF;",
+	NULL
+};
+
 /* table migrations: */
 /* If there is no migration table, we assume "version -1" -- the
  * version shipped with wrlinux 3.0, which had no version
@@ -173,6 +205,63 @@ static struct sql_migration {
 	{ NULL },
 };
 
+/* cleanup database before getting started
+ *
+ * On a large build, the logs database gets GIGANTIC...  And
+ * we rarely-if-ever delete things from it.  So instead of
+ * doing the vacuum operation on it at startup, which can impose
+ * a several-minute delay, we do it only on deletions.
+ *
+ * There's no setup for log database right now.
+ */
+char *file_setups[] = {
+	"VACUUM;",
+	NULL,
+};
+
+struct database_info {
+	char *pathname;
+	struct sql_index *indexes;
+	struct sql_table *tables;
+	struct sql_migration *migrations;
+	char **pragmas;
+	char **setups;
+	struct sqlite3 **db;
+};
+
+static struct database_info db_infos[] = {
+	{
+		"logs.db",
+		log_indexes,
+		log_tables,
+		log_migrations,
+		log_pragmas,
+		NULL,
+		&log_db
+	},
+	{
+		"files.db",
+		file_indexes,
+		file_tables,
+		file_migrations,
+		file_pragmas,
+		file_setups,
+#ifdef USE_MEMORY_DB
+		&real_file_db
+	},
+	{
+		":memory:",
+		file_indexes,
+		file_tables,
+		file_migrations,
+		file_pragmas,
+		file_setups,
+#endif
+		&file_db
+	},
+	{ 0, 0, 0, 0, 0, 0, 0 }
+};
+
 /* pretty-print error along with the underlying SQL error. */
 static void
 dberr(sqlite3 *db, char *fmt, ...) {
@@ -192,6 +281,67 @@ dberr(sqlite3 *db, char *fmt, ...) {
 	}
 }
 
+#ifdef USE_MEMORY_DB
+
+static void
+pdb_backup() {
+        sqlite3_backup *pBackup;
+        /* no point in doing this if we don't have a database to back up,
+	 * or nothing's changed.
+	 */
+        if (!file_db || !real_file_db || !file_db_dirty)
+                return;
+
+	pBackup = sqlite3_backup_init(real_file_db, "main", file_db, "main");
+	if (pBackup) {
+		int rc;
+		(void)sqlite3_backup_step(pBackup, -1);
+		rc = sqlite3_backup_finish(pBackup);
+		if (rc != SQLITE_OK) {
+			dberr(real_file_db, "error during flush to disk");
+		}
+	}
+	file_db_dirty = 0;
+}
+
+static void
+pdb_restore() {
+        sqlite3_backup *pBackup;
+        /* no point in doing this if we don't have a database to back up */
+        if (!file_db || !real_file_db)
+                return;
+
+	pBackup = sqlite3_backup_init(file_db, "main", real_file_db, "main");
+	if (pBackup) {
+		int rc;
+		(void)sqlite3_backup_step(pBackup, -1);
+		rc = sqlite3_backup_finish(pBackup);
+		if (rc != SQLITE_OK) {
+			dberr(file_db, "error during load from disk");
+		}
+	}
+	file_db_dirty = 0;
+}
+
+int
+pdb_maybe_backup(void) {
+        static int occasional = 0;
+        if (file_db && real_file_db) {
+                occasional = (occasional + 1) % 10;
+                if (occasional == 0) {
+                        pdb_backup();
+                        return 1;
+                }
+        }
+        return 0;
+}
+#else /* USE_MEMORY_DB */
+int
+pdb_maybe_backup(void) {
+	return 0;
+}
+#endif
+
 /* those who enjoy children, sausages, and databases, should not watch
  * them being made.
  */
@@ -235,7 +385,7 @@ make_tables(sqlite3 *db,
 		if (sql_tables[i].values) {
 			for (j = 0; sql_tables[i].values[j].fmt; ++j) {
 				char buffer[256];
-				sprintf(buffer, sql_tables[i].values[j].fmt, sql_tables[i].values[j].arg);
+				snprintf(buffer, sizeof(buffer), sql_tables[i].values[j].fmt, sql_tables[i].values[j].arg);
 				sql = sqlite3_mprintf("INSERT INTO %s ( %s ) VALUES ( %s );",
 					sql_tables[i].name,
 					*sql_tables[i].names,
@@ -377,131 +527,116 @@ make_tables(sqlite3 *db,
 static void
 cleanup_db(void) {
 	pseudo_debug(1, "server exiting\n");
+#ifdef USE_MEMORY_DB
+        if (real_file_db) {
+                pdb_backup();
+                sqlite3_close(real_file_db);
+	}
+#endif
 	if (file_db)
 		sqlite3_close(file_db);
 	if (log_db)
 		sqlite3_close(log_db);
 }
 
-/* I hate this function.
- * The need to separate logs and files into separate database (performance
- * and stability suffered when they were together) was discovered after
- * this was written, and it is still full of half-considered code and
- * unreasonable tests.  The test for whether db == &file_db is particularly
- * odious.
- *
- * The basic idea is to open the database, and make sure the tables exist
- * (using the make_tables function above).  Options are set to make sqlite
- * run reasonably efficiently.
+/* This function has been rewritten and I no longer hate it.  I just feel
+ * like it's important to say that.
  */
 static int
-get_db(sqlite3 **db) {
+get_db(struct database_info *dbinfo) {
 	int rc;
+	int i;
 	char *sql;
 	char **results;
 	int rows, columns;
 	char *errmsg;
 	static int registered_cleanup = 0;
 	char *dbfile;
+	sqlite3 *db;
 
-	if (!db)
+	if (!dbinfo)
 		return 1;
-	if (*db)
+	if (!dbinfo->db)
+		return 1;
+	/* this database is perhaps already initialized? */
+	if (*(dbinfo->db))
 		return 0;
-	if (db == &file_db) {
-		dbfile = pseudo_localstatedir_path("files.db");
-		rc = sqlite3_open(dbfile, db);
-#ifdef NPROFILE
-		sqlite3_profile(*db, xProfile, NULL);
+
+	dbfile = pseudo_localstatedir_path(dbinfo->pathname);
+#ifdef USE_MEMORY_DB
+        if (!strcmp(dbinfo->pathname, ":memory:")) {
+                rc = sqlite3_open(dbinfo->pathname, &db);
+        } else
 #endif
-		free(dbfile);
-	} else {
-		dbfile = pseudo_localstatedir_path("logs.db");
-		rc = sqlite3_open(dbfile, db);
-		free(dbfile);
-	}
+                rc = sqlite3_open(dbfile, &db);
+	free(dbfile);
 	if (rc) {
-		pseudo_diag("Failed: %s\n", sqlite3_errmsg(*db));
-		sqlite3_close(*db);
-		*db = NULL;
+		pseudo_diag("Failed: %s\n", sqlite3_errmsg(db));
+		sqlite3_close(db);
+		*(dbinfo->db) = NULL;
 		return 1;
 	}
+	/* we store this in the database_info, but hereafter we'll just use
+	 * the name db, because it is shorter.
+	 */
+	*dbinfo->db = db;
 	if (!registered_cleanup) {
 		atexit(cleanup_db);
 		registered_cleanup = 1;
 	}
-	if (db == &file_db) {
-		rc = sqlite3_exec(*db, "PRAGMA legacy_file_format = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "file_db legacy_file_format");
-		}
-		rc = sqlite3_exec(*db, "PRAGMA journal_mode = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "file_db journal_mode");
-		}
-		rc = sqlite3_exec(*db, "PRAGMA locking_mode = EXCLUSIVE;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "file_db locking_mode");
-		}
-		/* Setting this to NORMAL makes pseudo noticably slower
-		 * than fakeroot, but is perhaps more secure.  However,
-		 * note that sqlite always flushes to the OS; what is lacking
-		 * in non-synchronous mode is waiting for the OS to
-		 * confirm delivery to media, and also a bunch of cache
-		 * flushing and reloading which we probably don't really
-		 * need.
-		 */
-		rc = sqlite3_exec(*db, "PRAGMA synchronous = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "file_db synchronous");
-		}
-	} else if (db == &log_db) {
-		rc = sqlite3_exec(*db, "PRAGMA legacy_file_format = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "log_db legacy_file_format");
-		}
-		rc = sqlite3_exec(*db, "PRAGMA journal_mode = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "log_db journal_mode");
-		}
-		rc = sqlite3_exec(*db, "PRAGMA locking_mode = EXCLUSIVE;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "log_db locking_mode");
-		}
-		rc = sqlite3_exec(*db, "PRAGMA synchronous = OFF;", NULL, NULL, &errmsg);
-		if (rc) {
-			dberr(*db, "log_db synchronous");
+	if (dbinfo->pragmas) {
+		for (i = 0; dbinfo->pragmas[i]; ++i) {
+			rc = sqlite3_exec(db, dbinfo->pragmas[i], NULL, NULL, &errmsg);
+			if (rc) {
+				dberr(db, dbinfo->pragmas[i]);
+			}
 		}
 	}
 	/* create database tables or die trying */
 	sql =	"SELECT name FROM sqlite_master "
 		"WHERE type = 'table' "
 		"ORDER BY name;";
-	rc = sqlite3_get_table(*db, sql, &results, &rows, &columns, &errmsg);
+	rc = sqlite3_get_table(db, sql, &results, &rows, &columns, &errmsg);
 	if (rc) {
 		pseudo_diag("Failed: %s\n", errmsg);
 	} else {
-		if (db == &file_db) {
-			rc = make_tables(*db, file_tables, file_indexes, file_migrations, results, rows);
-		} else if (db == &log_db) {
-			rc = make_tables(*db, log_tables, log_indexes, log_migrations, results, rows);
-		}
+		rc = make_tables(db, dbinfo->tables, dbinfo->indexes, dbinfo->migrations, results, rows);
 		sqlite3_free_table(results);
 	}
-	/* cleanup database before getting started
-	 *
-	 * On a large build, the logs database gets GIGANTIC...  And
-	 * we rarely-if-ever delete things from it.  So instead of
-	 * doing the vacuum operation on it at startup, which can impose
-	 * a several-minute delay, we do it only on deletions.
-	 *
+	/* as of now, the only setup is a vacuum operation which we don't care about
+	 * the results of.
 	 */
-	if (db == &file_db) {
-		sqlite3_exec(*db, "VACUUM;", NULL, NULL, &errmsg);
+	if (dbinfo->setups) {
+		for (i = 0; dbinfo->setups[i]; ++i) {
+			sqlite3_exec(db, dbinfo->setups[i], NULL, NULL, &errmsg);
+		}
 	}
 	return rc;
 }
 
+static int
+get_dbs(void) {
+	int err = 0;
+	int i;
+#ifdef USE_MEMORY_DB
+        int already_loaded = 0;
+        if (file_db)
+                already_loaded = 1;
+#endif
+	for (i = 0; db_infos[i].db; ++i) {
+		if (get_db(&db_infos[i])) {
+			pseudo_diag("Error getting '%s' database.\n",
+				db_infos[i].pathname);
+			err = 1;
+		}
+	}
+#ifdef USE_MEMORY_DB
+        if (!already_loaded && file_db)
+                pdb_restore();
+#endif
+	return err;
+}
+
 /* put a prepared log entry into the database */
 int
 pdb_log_traits(pseudo_query_t *traits) {
@@ -509,8 +644,8 @@ pdb_log_traits(pseudo_query_t *traits) {
 	log_entry *e;
 	int rc;
 
-	if (!log_db && get_db(&log_db)) {
-		pseudo_diag("database error.\n");
+	if (!log_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 1;
 	}
 	e = calloc(sizeof(*e), 1);
@@ -607,8 +742,8 @@ pdb_log_entry(log_entry *e) {
 	int field;
 	int rc;
 
-	if (!log_db && get_db(&log_db)) {
-		pseudo_diag("database error.\n");
+	if (!log_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 1;
 	}
 
@@ -705,8 +840,8 @@ pdb_log_msg(pseudo_sev_t severity, pseudo_msg_t *msg, const char *program, const
 		text = buffer;
 	}
 
-	if (!log_db && get_db(&log_db)) {
-		pseudo_diag("database error.\n");
+	if (!log_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 1;
 	}
 
@@ -839,8 +974,8 @@ pdb_query(char *stmt_type, pseudo_query_t *traits, unsigned long fields, int uni
 	pseudo_query_field_t f;
 	static buffer *sql;
 
-	if (!log_db && get_db(&log_db)) {
-		pseudo_diag("database error.\n");
+	if (!log_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return NULL;
 	}
 
@@ -1046,6 +1181,7 @@ pdb_delete(pseudo_query_t *traits, unsigned long fields) {
 
 	/* no need to return it, so... */
 	if (stmt) {
+		file_db_dirty = 1;
 		int rc = sqlite3_step(stmt);
 		if (rc != SQLITE_DONE) {
 			dberr(log_db, "deletion failed");
@@ -1227,8 +1363,8 @@ pdb_link_file(pseudo_msg_t *msg) {
 		    " ( path, dev, ino, uid, gid, mode, rdev, deleting ) "
 		    " VALUES (?, ?, ?, ?, ?, ?, ?, 0);";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!insert) {
@@ -1256,6 +1392,7 @@ pdb_link_file(pseudo_msg_t *msg) {
 		(msg->pathlen ? msg->path : "<nil> (as NAMELESS FILE)"),
 		(unsigned long long) msg->dev, (unsigned long long) msg->ino,
 		(int) msg->mode, msg->uid);
+	file_db_dirty = 1;
 	rc = sqlite3_step(insert);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "insert may have failed (rc %d)", rc);
@@ -1272,8 +1409,8 @@ pdb_unlink_file_dev(pseudo_msg_t *msg) {
 	int rc;
 	char *sql = "DELETE FROM files WHERE dev = ? AND ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!sql_delete) {
@@ -1288,6 +1425,7 @@ pdb_unlink_file_dev(pseudo_msg_t *msg) {
 	}
 	sqlite3_bind_int(sql_delete, 1, msg->dev);
 	sqlite3_bind_int(sql_delete, 2, msg->ino);
+	file_db_dirty = 1;
 	rc = sqlite3_step(sql_delete);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "delete by inode may have failed");
@@ -1305,8 +1443,8 @@ pdb_update_file_path(pseudo_msg_t *msg) {
 	char *sql = "UPDATE files SET path = ? "
 		"WHERE path = 'NAMELESS FILE' and dev = ? AND ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!update) {
@@ -1323,6 +1461,7 @@ pdb_update_file_path(pseudo_msg_t *msg) {
 	sqlite3_bind_text(update, 1, msg->path, -1, SQLITE_STATIC);
 	sqlite3_bind_int(update, 2, msg->dev);
 	sqlite3_bind_int(update, 3, msg->ino);
+	file_db_dirty = 1;
 	rc = sqlite3_step(update);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "update path by inode may have failed");
@@ -1339,8 +1478,8 @@ pdb_may_unlink_file(pseudo_msg_t *msg, int deleting) {
 	int rc, exact;
 	char *sql_mark_file = "UPDATE files SET deleting = ? WHERE path = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!mark_file) {
@@ -1360,6 +1499,7 @@ pdb_may_unlink_file(pseudo_msg_t *msg, int deleting) {
 		pseudo_debug(1, "cannot mark a file for pending deletion without a path.");
 		return 1;
 	}
+	file_db_dirty = 1;
 	rc = sqlite3_step(mark_file);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "mark for deletion may have failed");
@@ -1383,8 +1523,8 @@ pdb_cancel_unlink_file(pseudo_msg_t *msg) {
 	int rc, exact;
 	char *sql_mark_file = "UPDATE files SET deleting = 0 WHERE path = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!mark_file) {
@@ -1403,6 +1543,7 @@ pdb_cancel_unlink_file(pseudo_msg_t *msg) {
 		pseudo_debug(1, "cannot unmark a file for pending deletion without a path.");
 		return 1;
 	}
+	file_db_dirty = 1;
 	rc = sqlite3_step(mark_file);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "unmark for deletion may have failed");
@@ -1423,8 +1564,8 @@ pdb_did_unlink_files(int deleting) {
 	int rc, exact;
 	char *sql_delete_exact = "DELETE FROM files WHERE deleting = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!delete_exact) {
@@ -1439,6 +1580,7 @@ pdb_did_unlink_files(int deleting) {
 		return 0;
 	}
 	sqlite3_bind_int(delete_exact, 1, deleting);
+	file_db_dirty = 1;
 	rc = sqlite3_step(delete_exact);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "cleanup of files marked for deletion may have failed");
@@ -1457,8 +1599,8 @@ pdb_did_unlink_file(char *path, int deleting) {
 	int rc, exact;
 	char *sql_delete_exact = "DELETE FROM files WHERE path = ? AND deleting = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!delete_exact) {
@@ -1474,6 +1616,7 @@ pdb_did_unlink_file(char *path, int deleting) {
 	}
 	sqlite3_bind_text(delete_exact, 1, path, -1, SQLITE_STATIC);
 	sqlite3_bind_int(delete_exact, 2, deleting);
+	file_db_dirty = 1;
 	rc = sqlite3_step(delete_exact);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "cleanup of file marked for deletion may have failed");
@@ -1492,8 +1635,8 @@ pdb_unlink_file(pseudo_msg_t *msg) {
 	int rc, exact;
 	char *sql_delete_exact = "DELETE FROM files WHERE path = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!delete_exact) {
@@ -1512,6 +1655,7 @@ pdb_unlink_file(pseudo_msg_t *msg) {
 		pseudo_debug(1, "cannot unlink a file without a path.");
 		return 1;
 	}
+	file_db_dirty = 1;
 	rc = sqlite3_step(delete_exact);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "delete exact by path may have failed");
@@ -1538,8 +1682,8 @@ pdb_unlink_contents(pseudo_msg_t *msg) {
 	char *sql_delete_sub = "DELETE FROM files WHERE "
 				"(path > (? || '/') AND path < (? || '0'));";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!delete_sub) {
@@ -1559,6 +1703,7 @@ pdb_unlink_contents(pseudo_msg_t *msg) {
 		pseudo_debug(1, "cannot unlink a file without a path.");
 		return 1;
 	}
+	file_db_dirty = 1;
 	rc = sqlite3_step(delete_sub);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "delete sub by path may have failed");
@@ -1587,8 +1732,8 @@ pdb_rename_file(const char *oldpath, pseudo_msg_t *msg) {
 	char *sql_update_sub = "UPDATE files SET path = replace(path, ?, ?) "
 			       "WHERE (path > (? || '/') AND path < (? || '0'));";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!update_exact) {
@@ -1626,6 +1771,7 @@ pdb_rename_file(const char *oldpath, pseudo_msg_t *msg) {
 
 	rc = sqlite3_exec(file_db, "BEGIN;", NULL, NULL, NULL);
 
+	file_db_dirty = 1;
 	rc = sqlite3_step(update_exact);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "update exact may have failed: rc %d", rc);
@@ -1656,8 +1802,8 @@ pdb_renumber_all(dev_t from, dev_t to) {
 		    " SET dev = ? "
 		    " WHERE dev = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!update) {
@@ -1676,6 +1822,7 @@ pdb_renumber_all(dev_t from, dev_t to) {
 		dberr(file_db, "error binding device numbers to update");
 	}
 
+	file_db_dirty = 1;
 	rc = sqlite3_step(update);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "update may have failed: rc %d", rc);
@@ -1697,8 +1844,8 @@ pdb_update_inode(pseudo_msg_t *msg) {
 		    " SET dev = ?, ino = ? "
 		    " WHERE path = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!update) {
@@ -1726,6 +1873,7 @@ pdb_update_inode(pseudo_msg_t *msg) {
 		dberr(file_db, "error binding %s to select", msg->path);
 	}
 
+	file_db_dirty = 1;
 	rc = sqlite3_step(update);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "update may have failed: rc %d", rc);
@@ -1749,8 +1897,8 @@ pdb_update_file(pseudo_msg_t *msg) {
 		    " SET uid = ?, gid = ?, mode = ?, rdev = ? "
 		    " WHERE dev = ? AND ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!update) {
@@ -1770,6 +1918,7 @@ pdb_update_file(pseudo_msg_t *msg) {
 	sqlite3_bind_int(update, 5, msg->dev);
 	sqlite3_bind_int(update, 6, msg->ino);
 
+	file_db_dirty = 1;
 	rc = sqlite3_step(update);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "update may have failed: rc %d", rc);
@@ -1789,8 +1938,8 @@ pdb_find_file_exact(pseudo_msg_t *msg) {
 	int rc;
 	char *sql = "SELECT * FROM files WHERE path = ? AND dev = ? AND ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!select) {
@@ -1840,8 +1989,8 @@ pdb_find_file_path(pseudo_msg_t *msg) {
 	int rc;
 	char *sql = "SELECT * FROM files WHERE path = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 1;
 	}
 	if (!select) {
@@ -1897,8 +2046,8 @@ pdb_get_file_path(pseudo_msg_t *msg) {
 	char *sql = "SELECT path FROM files WHERE dev = ? AND ino = ?;";
 	char *response;
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!select) {
@@ -1946,8 +2095,8 @@ pdb_find_file_dev(pseudo_msg_t *msg) {
 	int rc;
 	char *sql = "SELECT * FROM files WHERE dev = ? AND ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!select) {
@@ -1995,8 +2144,8 @@ pdb_find_file_ino(pseudo_msg_t *msg) {
 	int rc;
 	char *sql = "SELECT * FROM files WHERE ino = ?;";
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 	if (!select) {
@@ -2039,8 +2188,8 @@ pdb_file_list
 pdb_files(void) {
 	pdb_file_list l;
 
-	if (!file_db && get_db(&file_db)) {
-		pseudo_diag("database error.\n");
+	if (!file_db && get_dbs()) {
+		pseudo_diag("%s: database error.\n", __func__);
 		return 0;
 	}
 
